/* SplitLayout.css */
.split-layout {
    display: flex; /* Use Flexbox */
    justify-content: space-between; /* Space between the columns */
    align-items: stretch; /* Stretch columns to match height */

}

.left-column, .right-column {
    flex: 1; /* Equal width for both columns */
    padding: 20px; /* Add padding to the columns */
}

.u-flex {
    display: flex;
}

.u-row {
    flex-direction: row;
    padding-bottom: 1rem;
}
.u-column {
    flex-direction: column;
}

.u-row-reverse {
    flex-direction: row-reverse;
}

.u-space-between {
    justify-content: space-between;
}

.u-center {
    justify-content: center;
}

.u-inline-block {
    display: inline-block;
}

.link-container{
    display: inline-block;
    width : 8rem;
}

.popup-loc {
    position: relative;
}

.popup-container {
    background-color: white;
    position: absolute;
    display: inline-block;

    border: thin black solid;
    border-radius: 4px;

    padding: 1rem;
    width: 24rem;
}

.popup-container li {
    padding-bottom: 1rem;
}

.popup-button {
    width:2rem
}

.input {
    display: none;
}

.custom-file-upload {
    border: thin black solid;
    border-radius: 4px;
    cursor: pointer;
}

.file-name {
    display: inline-block;
    width: 32rem;
}
